@@ -1,11 +1,36 @@
 package Model::Envoy::Storage::DBIC;
 
-our $VERSION = '0.1.1';
+our $VERSION = '0.2.0';
+
+use Moose;
+use Scalar::Util 'blessed';
+use MooseX::ClassAttribute;
+
+extends 'Model::Envoy::Storage';
 
 =head1 Model::Envoy::Storage::DBIC
 
 A Moose Role that adds a DBIx::Class persistence layer to your Moose class
 
+=head2 Configuration
+
+    with 'Model::Envoy' => { storage => {
+        'DBIC' => {
+            schema => sub {
+                ... connect to database here ...
+            }
+        }
+    } };
+
+The only configuration option for this plugin is a 'schema' method that returns a
+C<DBIx::Class:Schema> based object with an open connection to the database.
+
+=head3 C<dbic()>
+
+This is a method you will need to implement in each C<Model::Envoy> object
+that use this storage plugin. It should return the name of the DBIx::Class ResultClass
+that your Model uses for database storage.
+
 =head2 Traits
 
 This role implements one trait:
@@ -45,45 +70,9 @@ maps to the join table in the database).
 
 =back
 
-=head2 Required Methods
+=head2 Plugin Methods
 
-There are two methods you will need to implement in a class that uses this role:
-
-=head3 dbic()
-
-This should return the name of the DBIx::Class ResultClass that your Model
-uses for database storage.
-
-=head3 _schema()
-
-This must return a DBIx::Class schema object for other methods to use when
-communicating with your database.
-
-=cut
-
-use Moose::Role;
-use Scalar::Util 'blessed';
-
-# The name of the DBIx::Class ResultClass is stored here:
-requires 'dbic';
-
-# Model needs to provide its own connection to the database:
-requires '_schema';
-
-# The actual ResultClass for the model object is stored here:
-has '_dbic_result',
-    is      => 'rw',
-    isa     => 'Maybe[Object]',
-    lazy    => 1,
-    default => sub {
-        my ( $self ) = @_;
-
-        return $self->_schema->resultset( $self->dbic )->new({});
-    };
-
-=head2 Methods
-
-=head3 new_from_db( $dbic_result, [$no_rel] )
+=head3 build( $dbic_result, [$no_rel] )
 
 Takes a DBIx::Class result object and, if it's class matches your class's dbic()
 method, attempts to build a new instance of your class based on the $dbic_result
@@ -95,27 +84,76 @@ from the database and the number of new class instances created.
 
 Returns the class instance if successful.
 
+=head3 save()
+
+Performs either an insert or an update for the model, depending on whether
+there is already a record for it in the database.
+
+Returns the calling object for convenient chaining.
+
+=head3 delete()
+
+Deletes the persistent copy of the current model from the database, if has
+been stored there.
+
+Returns nothing.
+
+=head3 in_storage()
+
+Uses DBIx::Class's internal mechanisms to determine if this model
+is tied to a record in the database.
+
+Returns a true value if it is, otherwise returns a false value.
+
 =cut
 
-sub new_from_db {
-    my ( $class, $db_result, $no_rel ) = @_;
+class_has 'schema' => (
+    is       => 'rw',
+    isa      => 'DBIx::Class::Schema',
+);
 
-    return undef unless $db_result;
+# The actual ResultClass for the model object is stored here:
+has '_dbic_result',
+    is      => 'rw',
+    isa     => 'Maybe[Object]',
+    lazy    => 1,
+    default => sub {
+        my ( $self ) = @_;
 
-    die "cannot create a $class from a $db_result"
-        unless blessed $db_result && $db_result->isa( $class->dbic );
+        return $self->schema->resultset( $self->model->dbic )->new({});
+    };
+
+
+sub configure {
+    my ( $class, $conf ) = @_;
+
+    $class->schema(
+        ref $conf->{schema} eq 'CODE' ? $conf->{schema}->() : $conf->{schema}
+    );
+
+    $conf->{_configured} = 1;
+}
+
+
+sub build {
+    my ( $class, $model_class, $db_result, $no_rel ) = @_;
+
+    return undef
+        unless $db_result
+            && blessed $db_result
+            && $db_result->isa( $model_class->dbic );
 
     my $data  = {};
 
-    my %relationships = map { $_->name => 1 } @{$class->_dbic_relationships};
+    my %relationships = map { $_->name => 1 } @{$class->_dbic_relationships($model_class)};
 
-    foreach my $attr ( grep { defined $db_result->$_ } map { $_->name } @{$class->_dbic_attrs} ) {
+    foreach my $attr ( grep { defined $db_result->$_ } map { $_->name } @{$class->_dbic_attrs($model_class)} ) {
 
         next if $no_rel && exists $relationships{$attr};
 
         if ( blessed $db_result->$attr && $db_result->$attr->isa('DBIx::Class::ResultSet') ) {
 
-            my $attribute  = $class->meta->find_attribute_by_name($attr);
+            my $attribute  = $model_class->meta->find_attribute_by_name($attr);
             my $class_attr = $attribute->meta->find_attribute_by_name('moose_class');
             my $factory    = $class_attr ? $class_attr->get_value($attribute) : undef;
 
@@ -123,7 +161,7 @@ sub new_from_db {
 
             if ( $factory ) {
 
-                $data->{$attr} = [ map { $factory->new_from_db( $_, 1 ) } $db_result->$attr ];
+                $data->{$attr} = [ map { $factory->build( $_, 1 ) } $db_result->$attr ];
             }
         }
         else {
@@ -132,24 +170,57 @@ sub new_from_db {
 
     }
 
-    return $class->new( _dbic_result => $db_result, %$data );
+    my $model = $model_class->new( %$data );
+    $model->get_storage( __PACKAGE__ )->_dbic_result( $db_result );
+
+    return $model;
 }
 
-=head3 db_save()
+sub fetch {
+    my $self        = shift;
+    my $model_class = shift;
+    my %params;
 
-Performs either an insert or an update for the model, depending on whether
-there is already a record for it in the database.
+    return undef unless @_;
 
-Returns the calling object for convenient chaining.
+    if ( @_ == 1 ) {
 
-=cut
+        my ( $id ) = @_;
 
-sub db_save {
+        $params{id} = $id;
+    }
+    else {
+
+        my ( $key, $value ) = @_;
+
+        $params{$key} = $value;
+    }
+
+    if ( my $result = ($self->schema->resultset( $model_class->dbic )
+        ->search(\%params))[0] ) {
+
+        return $model_class->build($result);
+    }
+
+    return undef;
+}
+
+sub list {
+    my $self = shift;
+    my $model_class = shift;
+
+    return [
+        map { $model_class->new_from_db($_) }
+            $self->schema->resultset( $self->model_class->dbic )->search(@_)
+    ];
+}
+
+sub save {
     my ( $self ) = @_;
 
     my $dbic_result = $self->_dbic_result;
 
-    $self->_schema->txn_do( sub {
+    $self->schema->txn_do( sub {
 
         # First update/insert non-relationships
         $self->_populate_dbic_result;
@@ -181,7 +252,7 @@ sub _db_save_relationship {
 
     my $name  = $attr->name;
     my $type  = $attr->meta->get_attribute('rel')->get_value($attr);
-    my $value = $self->_value_to_db( $self->$name );
+    my $value = $self->_value_to_db( $self->model->$name );
 
     if ( $type eq 'many_to_many' ) {
         my $setter = 'set_' . $name;
@@ -197,16 +268,7 @@ sub _db_save_relationship {
     }
 }
 
-=head3 db_delete()
-
-Deletes the persistent copy of the current model from the database, if has
-been stored there.
-
-Returns nothing.
-
-=cut
-
-sub db_delete {
+sub delete {
     my ( $self ) = @_;
 
     if ( $self->_dbic_result->in_storage ) {
@@ -216,15 +278,6 @@ sub db_delete {
     return;
 }
 
-=head3 in_storage()
-
-Uses DBIx::Class's internal mechanisms to determine if this model
-is tied to a record in the database.
-
-Returns a true value if it is, otherwise returns a false value.
-
-=cut
-
 sub in_storage {
     my ( $self ) = @_;
 
@@ -232,29 +285,35 @@ sub in_storage {
 }
 
 sub _dbic_attrs {
-    my ( $self ) = @_;
+    my ( $self, $model ) = @_;
+
+    $model //= $self->model;
 
     return [
         grep { $_->does('DBIC') }
-        $self->meta->get_all_attributes
+        $model->meta->get_all_attributes
     ];
 }
 
 sub _dbic_columns {
-    my ( $self ) = @_;
+    my ( $self, $model ) = @_;
+
+    $model //= $self->model;
 
     return [
         grep { $_->does('DBIC') && ! $_->is_relationship }
-        $self->meta->get_all_attributes
+        $model->meta->get_all_attributes
     ];
 }
 
 sub _dbic_relationships {
-    my ( $self ) = @_;
+    my ( $self, $model ) = @_;
+
+    $model //= $self->model;
 
     return [
         grep { $_->does('DBIC') && $_->is_relationship }
-        $self->meta->get_all_attributes
+        $model->meta->get_all_attributes
     ];
 }
 
@@ -264,7 +323,7 @@ sub _populate_dbic_result {
     for my $attr ( @{$self->_dbic_columns} ) {
 
         my $name  = $attr->name;
-        my $value = $self->_value_to_db( $self->$name );
+        my $value = $self->_value_to_db( $self->model->$name );
 
         $self->_dbic_result->$name( $value );
     }
@@ -277,10 +336,12 @@ sub _value_to_db {
 
             return [ map { $self->_value_to_db($_) } @$value ];
         }
-        elsif ( blessed $value && $value->can('does') && $value->does('Model::Envoy::Storage::DBIC') ) {
+        elsif ( blessed $value && $value->can('does') && $value->does('Model::Envoy') ) {
 
-            $value->_populate_dbic_result;
-            return $value->_dbic_result;
+            my $dbic = $value->get_storage(ref $self);
+
+            $dbic->_populate_dbic_result;
+            return $dbic->_dbic_result;
         }
 
         return $value;
